@@ -1,15 +1,14 @@
 import os
-import threading
 from pathlib import Path
-from typing import Optional
+from typing import Optional, List, Tuple
 
 import numpy as np
 import cv2 as cv
 
-from PySide6.QtWidgets import QFileDialog, QLabel, QLayout, QDialog, QProgressBar
-from PySide6.QtCore import Slot
+from PySide6.QtWidgets import QFileDialog, QLabel, QLayout, QDialog
+from PySide6.QtCore import QObject, Slot, Signal, QThreadPool, QRunnable
 
-from .api import Tat
+from .clustering import Tat
 from .checkable_image_entry import CheckableImageEntry
 from .preview_window import PreviewWindow
 from .cluster_image_entry import ClusterImageEntry
@@ -20,22 +19,77 @@ from .ui_progress_bar import Ui_ProgressBar
 from .utils import load_image, apply_colormap, create_cluster, array3d_to_pixmap
 
 
+class ClusteringSignals(QObject):
+    finished = Signal()
+    progress = Signal(int, tuple)
+
+
+class ClusteringWorker(QRunnable):
+    signals = ClusteringSignals()
+    is_running = False
+
+    def __init__(self, entries: List[CheckableImageEntry], cluster_count: int, run_count: int, max_iter_count: int,
+                 output_directory: str):
+        super(ClusteringWorker, self).__init__()
+        self.entries = entries
+        self.cluster_count = cluster_count
+        self.run_count = run_count
+        self.max_iter_count = max_iter_count
+        self.output_directory = output_directory
+
+    @Slot()
+    def run(self) -> None:
+        self.is_running = True
+        for index, ime in enumerate(self.entries):
+            if not self.is_running:
+                break
+
+            input_basename_no_ext = (lambda basename: basename[0:basename.rfind(".")])(os.path.basename(ime.image_path))
+            layers, cluster = Tat.generate_layers(np.asarray(cv.imread(ime.image_path, flags=cv.IMREAD_GRAYSCALE)),
+                                                  self.cluster_count, self.run_count, self.max_iter_count)
+
+            layers_data: List[LayerData] = []
+            for i, layer in enumerate(layers):
+                output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_layer_{i}")
+                output_image_path = f"{output_path_no_ext}.png"
+                output_array_path = f"{output_path_no_ext}.npy"
+                np.save(output_array_path, layer)
+                cv.imwrite(output_image_path, apply_colormap(layer, cv.COLORMAP_VIRIDIS))
+                layers_data.append(LayerData(output_image_path, output_array_path, layer_index=i))
+
+            output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_cluster")
+            output_image_path = f"{output_path_no_ext}.png"
+            output_array_path = f"{output_path_no_ext}.npy"
+
+            np.save(output_array_path, cluster)
+            cv.imwrite(output_image_path, apply_colormap(cluster, cv.COLORMAP_JET))
+
+            self.signals.progress.emit(index + 1,
+                                       (output_image_path, output_array_path, input_basename_no_ext, layers_data))
+        self.signals.finished.emit()
+
+    @Slot()
+    def interrupt(self) -> None:
+        self.is_running = False
+
+
 class MainWindow(PreviewWindow):
     def __init__(self):
         super(MainWindow, self).__init__(None)
         self.ui = Ui_MainWindow()
         self.ui.setupUi(self)
+        self.thread_pool = QThreadPool()
 
         self.input_directory = ""
         self.output_directory = ""
         self.merger_directory: Optional[str] = None
         self.editor_window: Optional[ClusterEditor] = None
 
-        self.__generated_images_entries: list[ClusterImageEntry] = []
+        self.__generated_images_entries: List[ClusterImageEntry] = []
 
         self.ui.buttonInputDir.clicked.connect(self.load_input_directory)
         self.ui.buttonOutputDir.clicked.connect(self.load_output_directory)
-        self.ui.buttonGenerate.clicked.connect(self.generate)
+        self.ui.buttonGenerate.clicked.connect(self.generate_handler)
         self.ui.buttonCheckUncheck.clicked.connect(self.select_deselect)
         self.ui.buttonClearGenerated.clicked.connect(self.clear_generated)
 
@@ -59,7 +113,7 @@ class MainWindow(PreviewWindow):
         self.editor_window.register_merge_handler(self.merge_layers)
         self.editor_window.show()
 
-    def merge_layers(self, layers_indices: list[int]) -> None:
+    def merge_layers(self, layers_indices: List[int]) -> None:
         """
         Merge all the specified layers
         :param layers_indices: A range of the layers to merge
@@ -68,13 +122,13 @@ class MainWindow(PreviewWindow):
         if len(layers_indices) == 0:
             return
 
-        merged_cluster_ime: list[ClusterImageEntry] = []
+        merged_cluster_ime: List[ClusterImageEntry] = []
 
         first = True
         while len(self.__generated_images_entries) > 0:
             ime: ClusterImageEntry = self.__generated_images_entries.pop(0)
             merged: Optional[np.ndarray] = None
-            parent_layers: list[int] = []
+            parent_layers: List[int] = []
             for i in layers_indices:
                 layer_data: LayerData = ime.get_layer_data(i)
                 if layer_data.is_merger:
@@ -145,10 +199,11 @@ class MainWindow(PreviewWindow):
 
     @Slot()
     def load_input_directory(self):
-        self.input_directory = QFileDialog.getExistingDirectory(self)
-        if len(self.input_directory) == 0:
+        in_dir = QFileDialog.getExistingDirectory(self)
+        if len(in_dir) == 0:
             return
 
+        self.input_directory = in_dir
         self.clear_image_entries()
         self.clear_preview_image()
 
@@ -156,7 +211,7 @@ class MainWindow(PreviewWindow):
 
         src_layout = self.source_layout()
         first = True
-        for entry in os.scandir(self.input_directory):
+        for entry in sorted(os.scandir(self.input_directory), key=lambda x: x.name):
             if not Tat.is_image(entry.path):
                 continue
 
@@ -175,10 +230,11 @@ class MainWindow(PreviewWindow):
 
     @Slot()
     def load_output_directory(self):
-        self.output_directory = QFileDialog.getExistingDirectory(self)
-        if len(self.output_directory) == 0:
+        out_dir = QFileDialog.getExistingDirectory(self)
+        if len(out_dir) == 0:
             return
 
+        self.output_directory = out_dir
         self.merger_directory = os.path.join(self.output_directory, "merged")
         Path(self.merger_directory).mkdir(exist_ok=True)
 
@@ -188,92 +244,42 @@ class MainWindow(PreviewWindow):
 
     @Slot()
     def generate_handler(self):
+        self.ui.buttonGenerate.setEnabled(False)
+
         progress_bar = QDialog(self)
         progress_bar.ui = Ui_ProgressBar()
         progress_bar.ui.setupUi(progress_bar)
 
-        container = self.ui.scrollAreaWidgetContentsDst.layout()
-        img_count = 0
-        for index, ime in enumerate(self._source_image_entries):
-            ime: CheckableImageEntry
-            if not ime.isChecked():
-                continue
-            thread = threading.Thread(target=self.generate_cluster, args=(ime, container, progress_bar.ui.progressBar))
-            thread.start()
-            img_count += 1
-
-        progress_bar.ui.progressBar.setMaximum(img_count)
-        progress_bar.show()
-
-    # WIP
-    def generate_cluster(self, ime: CheckableImageEntry, container: QLayout, progressbar: QProgressBar):
-        input_basename_no_ext = (lambda basename: basename[0:basename.rfind(".")])(os.path.basename(ime.image_path))
-        layers, cluster = Tat.generate_layers(np.asarray(cv.imread(ime.image_path, flags=cv.IMREAD_GRAYSCALE)),
-                                              self.ui.clusterCount.value(), self.ui.runCount.value(),
-                                              self.ui.maxIterCount.value())
-
-        layers_data: list[LayerData] = []
-        for i, layer in enumerate(layers):
-            output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_layer_{i}")
-            output_image_path = f"{output_path_no_ext}.png"
-            output_matrix_path = f"{output_path_no_ext}.npy"
-            np.save(output_matrix_path, layer)
-            cv.imwrite(output_image_path, apply_colormap(layer, cv.COLORMAP_VIRIDIS))
-            layers_data.append(LayerData(output_image_path, output_matrix_path, layer_index=i))
-
-        output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_cluster")
-        output_image_path = f"{output_path_no_ext}.png"
-        output_array_path = f"{output_path_no_ext}.pny"
-
-        np.save(output_array_path, cluster)
-        cv.imwrite(output_image_path, apply_colormap(cluster, cv.COLORMAP_JET))
-
-        qim = load_image(output_image_path)
-        ime = ClusterImageEntry(container.parent(), qim, output_image_path, output_array_path, input_basename_no_ext,
-                                layers_data)
-        ime.registerMousePressHandler(self.image_entry_click_handler)
-        ime.register_mouse_double_click_action(self.open_preview_window)
-        container.addWidget(ime)
-        self.__generated_images_entries.append(ime)
-        progressbar.setValue(progressbar.value() + 1)
-
-    @Slot()
-    def generate(self):
-        layout = self.ui.scrollAreaWidgetContentsDst.layout()
-        first = True
-        for ime in self._source_image_entries:
-            if not ime.isChecked() or not Tat.is_image(ime.image_path):
-                continue
-
-            input_basename_no_ext = (lambda basename: basename[0:basename.rfind(".")])(os.path.basename(ime.image_path))
-            layers, cluster = Tat.generate_layers(np.asarray(cv.imread(ime.image_path, flags=cv.IMREAD_GRAYSCALE)),
-                                                  self.ui.clusterCount.value(), self.ui.runCount.value(),
-                                                  self.ui.maxIterCount.value())
-
-            layers_data: list[LayerData] = []
-            for i, layer in enumerate(layers):
-                output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_layer_{i}")
-                output_image_path = f"{output_path_no_ext}.png"
-                output_array_path = f"{output_path_no_ext}.npy"
-                np.save(output_array_path, layer)
-                cv.imwrite(output_image_path, apply_colormap(layer, cv.COLORMAP_VIRIDIS))
-                layers_data.append(LayerData(output_image_path, output_array_path, layer_index=i))
-
-            output_path_no_ext = os.path.join(self.output_directory, f"{input_basename_no_ext}_cluster")
-            output_image_path = f"{output_path_no_ext}.png"
-            output_array_path = f"{output_path_no_ext}.npy"
-
-            np.save(output_array_path, cluster)
-            cv.imwrite(output_image_path, apply_colormap(cluster, cv.COLORMAP_JET))
-
-            qim = load_image(output_image_path)
-            ime = ClusterImageEntry(layout.parent(), qim, output_image_path, output_array_path, input_basename_no_ext,
+        @Slot()
+        def add_cluster_image(progress: int, data: Tuple[str, str, str, List[LayerData]]):
+            image_path, array_path, name, layers_data = data
+            container: QLayout = self.ui.scrollAreaWidgetContentsDst.layout()
+            ime = ClusterImageEntry(container.parent(), load_image(image_path), image_path, array_path, name,
                                     layers_data)
             ime.registerMousePressHandler(self.image_entry_click_handler)
             ime.register_mouse_double_click_action(self.open_preview_window)
-            layout.addWidget(ime)
+            container.addWidget(ime)
+            if len(self.__generated_images_entries) == 0:
+                self.set_preview_image(load_image(ime.image_path), ime)
+            progress_bar.ui.progressBar.setValue(progress)
             self.__generated_images_entries.append(ime)
 
-            if first:
-                self.set_preview_image(qim, ime)
-                first = False
+        selected_entries = self.get_selected_entries()
+        worker = ClusteringWorker(selected_entries, self.ui.clusterCount.value(), self.ui.runCount.value(),
+                                  self.ui.maxIterCount.value(), self.output_directory)
+
+        @Slot()
+        def finished_generating():
+            progress_bar.close()
+            worker.signals.progress.disconnect(add_cluster_image)
+            worker.signals.finished.disconnect(finished_generating)
+            self.ui.buttonGenerate.setEnabled(True)
+
+        worker.signals.progress.connect(add_cluster_image)
+        worker.signals.finished.connect(finished_generating)
+        progress_bar.ui.cancelButton.clicked.connect(worker.interrupt)
+
+        progress_bar.ui.progressBar.setMaximum(len(selected_entries))
+        progress_bar.show()
+
+        self.thread_pool.start(worker)
